@@ -19,6 +19,7 @@ import logging
 import os
 import socket
 from threading import Event, Lock, Thread
+import weakref
 
 from six import BytesIO
 
@@ -46,81 +47,175 @@ except ImportError:
 log = logging.getLogger(__name__)
 
 
-class LibevConnection(Connection):
-    """
-    An implementation of :class:`.Connection` that uses libev for its event loop.
-    """
-    _loop = libev.Loop()
-    _loop_notifier = libev.Async(_loop)
-    _loop_notifier.start()
+def _cleanup(loop_weakref):
+    try:
+        loop = loop_weakref()
+    except ReferenceError:
+        return
 
-    # prevent _loop_notifier from keeping the loop from returning
-    _loop.unref()
+    loop._cleanup()
 
-    _loop_started = None
-    _loop_lock = Lock()
-    _loop_shutdown = False
 
-    @classmethod
-    def _run_loop(cls):
+class LibevLoop(object):
+
+    def __init__(self):
+        self._loop = libev.Loop()
+        self._notifier = libev.Async(self._loop)
+        self._notifier.start()
+
+        # prevent _notifier from keeping the loop from returning
+        self._loop.unref()
+
+        self._started = False
+        self._shutdown = False
+        self._lock = Lock()
+
+        self._thread = None
+
+        # set of all connections; only replaced with a new copy
+        # while holding _conn_set_lock, never modified in place
+        self._live_conns = set()
+        # newly created connections that need their write/read watcher started
+        self._new_conns = set()
+        # recently closed connections that need their write/read watcher stopped
+        self._closed_conns = set()
+        self._conn_set_lock = Lock()
+
+        self._preparer = libev.Prepare(self._loop, self._loop_will_run)
+        # prevent _preparer from keeping the loop from returning
+        self._loop.unref()
+        self._preparer.start()
+
+        atexit.register(partial(_cleanup, weakref.ref(self)))
+
+    def notify(self):
+        self._notifier.send()
+
+    def maybe_start(self):
+        should_start = False
+        with self._lock:
+            if not self._started:
+                log.debug("Starting libev event loop")
+                self._started = True
+                should_start = True
+
+        if should_start:
+            self._thread = Thread(target=self._run_loop, name="event_loop")
+            self._thread.daemon = True
+            self._thread.start()
+
+        self._notifier.send()
+
+    def _run_loop(self):
         while True:
-            end_condition = cls._loop.start()
+            end_condition = self._loop.start()
             # there are still active watchers, no deadlock
-            with cls._loop_lock:
-                if not cls._loop_shutdown and (end_condition or cls._live_conns):
+            with self._lock:
+                if not self._shutdown and (end_condition or self._live_conns):
                     log.debug("Restarting event loop")
                     continue
                 else:
                     # all Connections have been closed, no active watchers
                     log.debug("All Connections currently closed, event loop ended")
-                    cls._loop_started = False
+                    self._started = False
                     break
 
-    @classmethod
-    def _maybe_start_loop(cls):
-        should_start = False
-        with cls._loop_lock:
-            if not cls._loop_started:
-                log.debug("Starting libev event loop")
-                cls._loop_started = True
-                should_start = True
+    def _cleanup(self):
+        self._shutdown = True
+        if not self._thread:
+            return
 
-        if should_start:
-            t = Thread(target=cls._run_loop, name="event_loop")
-            t.daemon = True
-            t.start()
-            atexit.register(partial(cls._cleanup, t))
-
-        return should_start
-
-    @classmethod
-    def _cleanup(cls, thread):
-        cls._loop_shutdown = True
         log.debug("Waiting for event loop thread to join...")
-        thread.join(timeout=1.0)
-        if thread.is_alive():
+        self._thread.join(timeout=1.0)
+        if self._thread.is_alive():
             log.warning(
                 "Event loop thread could not be joined, so shutdown may not be clean. "
                 "Please call Cluster.shutdown() to avoid this.")
 
         log.debug("Event loop thread was joined")
 
-    # class-level set of all connections; only replaced with a new copy
-    # while holding _conn_set_lock, never modified in place
-    _live_conns = set()
-    # newly created connections that need their write/read watcher started
-    _new_conns = set()
-    # recently closed connections that need their write/read watcher stopped
-    _closed_conns = set()
-    _conn_set_lock = Lock()
+    def connection_created(self, conn):
+        with self._conn_set_lock:
+            new_live_conns = self._live_conns.copy()
+            new_live_conns.add(conn)
+            self._live_conns = new_live_conns
 
+            new_new_conns = self._new_conns.copy()
+            new_new_conns.add(conn)
+            self._new_conns = new_new_conns
+
+    def connection_destroyed(self, conn):
+        with self._conn_set_lock:
+            new_live_conns = self._live_conns.copy()
+            new_live_conns.discard(conn)
+            self._live_conns = new_live_conns
+
+            new_closed_conns = self._closed_conns.copy()
+            new_closed_conns.add(conn)
+            self._closed_conns = new_closed_conns
+
+        self._notifier.send()
+
+    def _loop_will_run(self, prepare):
+        changed = False
+        for conn in self._live_conns:
+            if not conn.deque and conn._write_watcher_is_active:
+                if conn._write_watcher:
+                    conn._write_watcher.stop()
+                conn._write_watcher_is_active = False
+                changed = True
+            elif conn.deque and not conn._write_watcher_is_active:
+                conn._write_watcher.start()
+                conn._write_watcher_is_active = True
+                changed = True
+
+        if self._new_conns:
+            with self._conn_set_lock:
+                to_start = self._new_conns
+                self._new_conns = set()
+
+            for conn in to_start:
+                conn._read_watcher.start()
+
+            changed = True
+
+        if self._closed_conns:
+            with self._conn_set_lock:
+                to_stop = self._closed_conns
+                self._closed_conns = set()
+
+            for conn in to_stop:
+                if conn._write_watcher:
+                    conn._write_watcher.stop()
+                    # clear reference cycles from IO callback
+                    del conn._write_watcher
+                if conn._read_watcher:
+                    conn._read_watcher.stop()
+                    # clear reference cycles from IO callback
+                    del conn._read_watcher
+
+            changed = True
+
+        if changed:
+            self._notifier.send()
+
+
+class LibevConnection(Connection):
+    """
+    An implementation of :class:`.Connection` that uses libev for its event loop.
+    """
+    _libevloop = None
     _write_watcher_is_active = False
-
     _total_reqd_bytes = 0
     _read_watcher = None
     _write_watcher = None
     _socket = None
 
+    @classmethod
+    def initialize_reactor(cls):
+        if not cls._libevloop:
+            cls._libevloop = LibevLoop()
+
     @classmethod
     def factory(cls, *args, **kwargs):
         timeout = kwargs.pop('timeout', 5.0)
@@ -134,68 +229,6 @@ class LibevConnection(Connection):
         else:
             return conn
 
-    @classmethod
-    def _connection_created(cls, conn):
-        with cls._conn_set_lock:
-            new_live_conns = cls._live_conns.copy()
-            new_live_conns.add(conn)
-            cls._live_conns = new_live_conns
-
-            new_new_conns = cls._new_conns.copy()
-            new_new_conns.add(conn)
-            cls._new_conns = new_new_conns
-
-    @classmethod
-    def _connection_destroyed(cls, conn):
-        with cls._conn_set_lock:
-            new_live_conns = cls._live_conns.copy()
-            new_live_conns.discard(conn)
-            cls._live_conns = new_live_conns
-
-            new_closed_conns = cls._closed_conns.copy()
-            new_closed_conns.add(conn)
-            cls._closed_conns = new_closed_conns
-
-    @classmethod
-    def loop_will_run(cls, prepare):
-        changed = False
-        for conn in cls._live_conns:
-            if not conn.deque and conn._write_watcher_is_active:
-                if conn._write_watcher:
-                    conn._write_watcher.stop()
-                conn._write_watcher_is_active = False
-                changed = True
-            elif conn.deque and not conn._write_watcher_is_active:
-                conn._write_watcher.start()
-                conn._write_watcher_is_active = True
-                changed = True
-
-        if cls._new_conns:
-            with cls._conn_set_lock:
-                to_start = cls._new_conns
-                cls._new_conns = set()
-
-            for conn in to_start:
-                conn._read_watcher.start()
-
-            changed = True
-
-        if cls._closed_conns:
-            with cls._conn_set_lock:
-                to_stop = cls._closed_conns
-                cls._closed_conns = set()
-
-            for conn in to_stop:
-                if conn._write_watcher:
-                    conn._write_watcher.stop()
-                if conn._read_watcher:
-                    conn._read_watcher.stop()
-
-            changed = True
-
-        if changed:
-            cls._loop_notifier.send()
-
     def __init__(self, *args, **kwargs):
         Connection.__init__(self, *args, **kwargs)
 
@@ -219,17 +252,16 @@ class LibevConnection(Connection):
             for args in self.sockopts:
                 self._socket.setsockopt(*args)
 
-        with self._loop_lock:
-            self._read_watcher = libev.IO(self._socket.fileno(), libev.EV_READ, self._loop, self.handle_read)
-            self._write_watcher = libev.IO(self._socket.fileno(), libev.EV_WRITE, self._loop, self.handle_write)
+        with self._libevloop._lock:
+            self._read_watcher = libev.IO(self._socket.fileno(), libev.EV_READ, self._libevloop._loop, self.handle_read)
+            self._write_watcher = libev.IO(self._socket.fileno(), libev.EV_WRITE, self._libevloop._loop, self.handle_write)
 
         self._send_options_message()
 
-        self.__class__._connection_created(self)
+        self._libevloop.connection_created(self)
 
         # start the global event loop if needed
-        self._maybe_start_loop()
-        self._loop_notifier.send()
+        self._libevloop.maybe_start()
 
     def close(self):
         with self.lock:
@@ -238,9 +270,9 @@ class LibevConnection(Connection):
             self.is_closed = True
 
         log.debug("Closing connection (%s) to %s", id(self), self.host)
-        self.__class__._connection_destroyed(self)
-        self._loop_notifier.send()
+        self._libevloop.connection_destroyed(self)
         self._socket.close()
+        log.debug("Closed socket to %s", self.host)
 
         # don't leave in-progress operations hanging
         if not self.is_defunct:
@@ -349,7 +381,7 @@ class LibevConnection(Connection):
 
         with self._deque_lock:
             self.deque.extend(chunks)
-            self._loop_notifier.send()
+            self._libevloop.notify()
 
     def register_watcher(self, event_type, callback, register_timeout=None):
         self._push_watchers[event_type].add(callback)
@@ -361,9 +393,3 @@ class LibevConnection(Connection):
             self._push_watchers[event_type].add(callback)
         self.wait_for_response(
             RegisterMessage(event_list=type_callback_dict.keys()), timeout=register_timeout)
-
-
-_preparer = libev.Prepare(LibevConnection._loop, LibevConnection.loop_will_run)
-# prevent _preparer from keeping the loop from returning
-LibevConnection._loop.unref()
-_preparer.start()
